package osc.petra.logic;

import java.util.ArrayList;
import java.util.List;

import osc.petra.dataModel.GPS;
import osc.petra.dataModel.Paret;
import osc.petra.dataModel.Sector;
import osc.petra.dataModel.Via;
import osc.petra.dataModel.Zona;

public class ProgramControl implements ProgramControlInterface {
	
	private Query query;
	private List<Zona> zonaList;
	private RemoteDataInterface remoteData;
	private Via viaActual;
	
	public ProgramControl(){
		this.zonaList = new ArrayList<Zona>();
		this.remoteData = new RemoteData();
	}

	
	@Override
	public void executeQuery() throws Exception {
		this.zonaList = this.remoteData.executeQuery(this.query);
	}

	@Override
	public Zona getZona(int index) {
		return this.zonaList.get(index);
	}

	@Override
	public void loadAllZonaAndSector() throws Exception {
		this.zonaList = this.remoteData.loadAllZonaAndSector();
	}

	@Override
	public void loadViaActual() throws Exception {
		this.remoteData.loadVia(this.viaActual);
	}

	@Override
	public void newQuery(GPS punt, double radi) {
		QueryGPS q = new QueryGPS();
		q.addArea(punt, radi);
		this.query = q;
	}

	@Override
	public void newQuery(boolean totesZones) {
		if(totesZones) this.query = new Query();
		else this.query = new QueryZona();
	}

	@Override
	public int zonaCount() {
		return this.zonaList.size();
	}


	@Override
	public Query getQuery() {
		return this.query;
	}


	@Override
	public List<String> getListSectors(String zona) {
		ArrayList<String> sectors = new ArrayList<String>();
		
		Zona z = this.getZona(zona);
		if(z==null) return sectors;
		
		for(int i=0; i<z.componentNumber(); i++){
			sectors.add(z.getComponent(i).getName());
		}
		
		return sectors;
	}


	@Override
	public List<Via> getListVies(String zona, String sector) {
		ArrayList<Via> vies = new ArrayList<Via>();
		
		//trobar zona
		Zona z = this.getZona(zona);
		
		//si zona no trobada, retornem llista buida
		if(z==null) return vies;
		
		//trobar sector
		Sector s = null;
		boolean trobat = false;
		for(int i=0; i<z.componentNumber(); i++){
			if(z.getComponent(i).getName().equals(sector)){
				trobat = true;
				s = z.getComponent(i);
			}
		}
		
		//si sector no trobat, retornem llista buida
		if(!trobat) return vies;
		
		//per cada paret del sector obtenim llista vies
		for(int i=0; i<s.componentNumber(); i++){
			Paret p = s.getComponent(i);
			for(int j=0; j<p.componentNumber(); j++){
				vies.add(p.getComponent(j));
			}
		}
		
		return vies;
	}


	@Override
	public List<String> getListZones() {
		ArrayList<String> zones = new ArrayList<String>();
		for(Zona z : this.zonaList){
			zones.add(z.getName());
		}
		return zones;
	}
	
	
	private Zona getZona(String zona){
		Zona z = null;
		boolean trobat = false;
		
		for(int i=0; i<this.zonaCount() && !trobat; i++){
			if(this.zonaList.get(i).getName().equals(zona)){
				trobat = true;
				z = this.zonaList.get(i);
			}
		}
		
		return z;
	}


	@Override
	public Via getViaActual() {
		return this.viaActual;
	}


	@Override
	public void setViaActual(Via via) {
		this.viaActual = via;
	}
}
