package osc.petra.logic;

import java.io.BufferedInputStream;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.util.Log;

import osc.petra.dataModel.Allotjament;
import osc.petra.dataModel.GPS;
import osc.petra.dataModel.Paret;
import osc.petra.dataModel.Parquing;
import osc.petra.dataModel.Sector;
import osc.petra.dataModel.Via;
import osc.petra.dataModel.Zona;

public class RemoteData implements RemoteDataInterface {
	private static final String ipServer = "192.168.2.11";
	private static final String adress = "http://"+ipServer+":8080/exist/xquery/";

	@Override
	public List<Zona> executeQuery(Query query) throws Exception {
		ArrayList<Zona> list = new ArrayList<Zona>();
		
		if(query instanceof QueryZona){//Zona-Sector
			executeQueryZonaSector((QueryZona)query, list);
		}else if(query instanceof QueryGPS){//GPS o Maps
			executeQueryGPS((QueryGPS)query, list);
		}else{//A totes Zones i Sectors
			executeQueryTotesZonesSectors(query, list);
		}
		
		return list;
	}

	private void executeQueryTotesZonesSectors(Query query, ArrayList<Zona> list) throws Exception {
		String sQuery = adress+"viestotssectors.xql?";
		
		if (query.hasEscalada()) sQuery+="escalada="+query.getEscalada()+"&";
		if(query.hasTipus()) sQuery+="tipus="+query.getTipus()+"&";
		if(query.hasDificultatMaxima()) sQuery+="dmaxima="+query.getDificultatMaxima()+"&";
		if(query.hasOrientacio()) sQuery+="orientacio="+query.getOrientacio()+"&";
		if(query.hasLlargs()) sQuery+="llargs="+query.getLlargs()+"&";
		if(query.hasRoca()) sQuery+="roca="+query.getRoca()+"&";
		
		sQuery = sQuery.substring(0, sQuery.length()-1);
		Log.d("CONSULTA", sQuery);
		this.executeQuery(sQuery, list);
	}

	
	
	private void executeQueryGPS(QueryGPS query, ArrayList<Zona> list) throws Exception {
		String sQuery = adress+"viesgps.xql?";
		
		double[] rangLatitud = query.getRangLatitud();
		sQuery += "latitudMin="+rangLatitud[0]+"&";
		sQuery += "latitudMax="+rangLatitud[1]+"&";
		
		double[] rangLongitud = query.getRangLongitud();
		sQuery += "longitudMin="+rangLongitud[0]+"&";
		sQuery += "longitudMax="+rangLongitud[1]+"&";
		
		if (query.hasEscalada()) sQuery+="escalada="+query.getEscalada()+"&";
		if(query.hasTipus()) sQuery+="tipus="+query.getTipus()+"&";
		if(query.hasDificultatMaxima()) sQuery+="dmaxima="+query.getDificultatMaxima()+"&";
		if(query.hasOrientacio()) sQuery+="orientacio="+query.getOrientacio()+"&";
		if(query.hasLlargs()) sQuery+="llargs="+query.getLlargs()+"&";
		if(query.hasRoca()) sQuery+="roca="+query.getRoca()+"&";
		
		sQuery = sQuery.substring(0, sQuery.length()-1);
		Log.d("CONSULTA", sQuery);
		this.executeQuery(sQuery, list);
	}
	
	

	private void executeQueryZonaSector(QueryZona query, ArrayList<Zona> list) throws Exception {
		String sQuery = adress+"vieszonasector.xql?zonasector=";
		
		String zs = "";
		for(int i=0; i<query.getCountZones(); i++){
			zs += "#"+query.getZona(i);
			for(int j=0; j<query.getCountSectors(i); j++){
				zs += ":"+query.getSector(i, j);
			}
		}
		sQuery += Uri.encode(zs);
		sQuery += "&";
		
		if (query.hasEscalada()) sQuery+="escalada="+query.getEscalada()+"&";
		if(query.hasTipus()) sQuery+="tipus="+query.getTipus()+"&";
		if(query.hasDificultatMaxima()) sQuery+="dmaxima="+query.getDificultatMaxima()+"&";
		if(query.hasOrientacio()) sQuery+="orientacio="+query.getOrientacio()+"&";
		if(query.hasLlargs()) sQuery+="llargs="+query.getLlargs()+"&";
		if(query.hasRoca()) sQuery+="roca="+query.getRoca()+"&";
		
		sQuery = sQuery.substring(0, sQuery.length()-1);
		this.executeQuery(sQuery, list);
	}
	
	
	
	private void executeQuery(String sQuery, ArrayList<Zona> list) throws Exception {
		Log.d("RemoteData - Query", sQuery);
		
		HttpURLConnection urlConnection = null;
		InputStream in = null;
		Document doc = null;
		
		try{
			URL url = new URL(sQuery);
			urlConnection = (HttpURLConnection) url.openConnection();
			in = urlConnection.getInputStream();
			doc = this.getXmlDocument(in);
		}catch(Exception e){
			Log.e("RemoteData", "executeQuery: "+e.getMessage());
			throw new Exception("No s'ha pogut accedir a les dades del servidor.");
		}
			
		try{
			this.parseViaList(doc, list);
		}catch(Exception e){
			Log.e("RemoteData - parser ", "executeQuery: " + e.getMessage());
			throw new Exception("Error de parser - La informació no s'ha rebut correctament.");
		}
		
		try{
			in.close();
			urlConnection.disconnect();
		}catch(Exception e){}
		
	}

	
	

	@Override
	public List<Zona> loadAllZonaAndSector() throws Exception {
		ArrayList<Zona> list = new ArrayList<Zona>();
		HttpURLConnection urlConnection = null;
		InputStream in = null;
		Document doc = null;
		try{
			URL url = new URL(adress+"zonasector.xql");
			Log.d("RemoteData - loadAllZonaAndSector", adress+"zonasector.xql");
			urlConnection = (HttpURLConnection) url.openConnection();
			in = urlConnection.getInputStream();
			doc = this.getXmlDocument(in);
		}catch(Exception e){
			Log.e("RemoteData - internet", "loadAllZonaAndSector: "+e.getMessage());
			throw new Exception("No s'ha pogut accedir a les dades del servidor.");
		}
		
		try{
			this.parseZonaSector(doc, list);
		}catch(Exception e){
			Log.e("RemoteData - parser ", "loadAllZonaAndSector: " + e.getMessage());
			throw new Exception("Error de parser - La informació no s'ha rebut correctament.");
		}
		
		try{
			in.close();
			urlConnection.disconnect();
		}catch(Exception e){}
		
		return list;
	}

	

	@Override
	public void loadVia(Via via) throws Exception {
		String params = "";
		
		params += "zona="+via.getZona()+"&";
		params += "sector="+via.getSector()+"&";
		params += "paret="+via.getParet()+"&";
		params += "via="+via.getNom();
		
		String sQuery = adress+"via.xql?"+params;
		Log.d("LOAD VIA", sQuery);
		
		HttpURLConnection urlConnection = null;
		InputStream in = null;
		Document doc = null;
		
		try{
			URL url = new URL(sQuery);
			
			urlConnection = (HttpURLConnection) url.openConnection();
			in = urlConnection.getInputStream();
			doc = this.getXmlDocument(in);
		}catch(Exception e){
			Log.e("RemoteData", "loadVia: "+e.getMessage());
			throw new Exception("No s'ha pogut accedir a les dades del servidor.");
		}
			
		try{
			this.parseVia(doc, via);
			via.setCarregada(true);
		}catch(Exception e){
			Log.e("RemoteData - parser ", "loadVia: " + e.getMessage());
			throw new Exception("Error de parser - La informació no s'ha rebut correctament.");
		}
			
		try{
			in.close();
			urlConnection.disconnect();
		}catch(Exception e){}
	}

	private void parseVia(Document doc, Via via) throws Exception {
		NodeList listGPS = doc.getElementsByTagName("gps");
		Element gps = null;
		for(int i=0; i<listGPS.getLength(); i++){
			if(listGPS.item(i).getParentNode().getNodeName().equals("via")){
				gps = (Element)listGPS.item(i);
			}
		}
		double latitud  = Double.parseDouble(gps.getElementsByTagName("latitud").item(0).getTextContent());
		double longitud = Double.parseDouble(gps.getElementsByTagName("longitud").item(0).getTextContent());
		via.setLocalitzacioGPS(new GPS(latitud,longitud));
		via.setLlargs(Integer.parseInt(doc.getElementsByTagName("llargs").item(0).getTextContent()));
		via.setEscalada(doc.getElementsByTagName("escalada").item(0).getTextContent());
		via.setTipus(doc.getElementsByTagName("tipus").item(0).getTextContent());
		via.setOrientacio(doc.getElementsByTagName("orientacio").item(0).getTextContent());
		via.setRoca(doc.getElementsByTagName("roca").item(0).getTextContent());
		via.setAproximacio(doc.getElementsByTagName("aproximacio").item(0).getTextContent());
		via.setDescens(doc.getElementsByTagName("descens").item(0).getTextContent());
		via.setEsquema(this.loadEsquema(doc.getElementsByTagName("esquema").item(0).getTextContent()));
		via.setUrlEsquema(doc.getElementsByTagName("esquema").item(0).getTextContent());
		
		NodeList material = doc.getElementsByTagName("mat");
		for(int i=0; i<material.getLength(); i++){
			via.addMaterial(material.item(i).getTextContent());
		}
		
		NodeList equipament = doc.getElementsByTagName("ancoratge");
		for(int i=0; i<equipament.getLength(); i++){
			via.addEquipament(equipament.item(i).getTextContent());
		}
		
		NodeList links = doc.getElementsByTagName("enllac");
		for(int i=0; i<links.getLength(); i++){
			via.addLink(links.item(i).getTextContent());
		}
		
		Element parquing = (Element)doc.getElementsByTagName("parquing").item(0);
		gps = (Element)parquing.getElementsByTagName("gps").item(0);
		latitud  = Double.parseDouble(gps.getElementsByTagName("latitud").item(0).getTextContent());
		longitud = Double.parseDouble(gps.getElementsByTagName("longitud").item(0).getTextContent());
		String acces   = parquing.getElementsByTagName("acces").item(0).getTextContent();
		String descrip = parquing.getElementsByTagName("descripcio").item(0).getTextContent();
		via.setParquing(new Parquing(new GPS(latitud, longitud), acces, descrip));
		
		NodeList allotjaments = doc.getElementsByTagName("allotjament");
		for(int i=0; i<allotjaments.getLength(); i++){
			Element a = (Element)allotjaments.item(i);
			gps = (Element)a.getElementsByTagName("gps").item(0);
			latitud  = Double.parseDouble(gps.getElementsByTagName("latitud").item(0).getTextContent());
			longitud = Double.parseDouble(gps.getElementsByTagName("longitud").item(0).getTextContent());
			
			String adreca = Allotjament.NO_DATA;
			String telf   = Allotjament.NO_DATA;
			String mail   = Allotjament.NO_DATA;
			
			NodeList posibleElement = a.getElementsByTagName("adreca");
			if(posibleElement.getLength()>0) adreca = posibleElement.item(0).getTextContent();
			
			posibleElement = a.getElementsByTagName("telefon");
			if(posibleElement.getLength()>0) telf = posibleElement.item(0).getTextContent();
			
			posibleElement = a.getElementsByTagName("mail");
			if(posibleElement.getLength()>0) mail = posibleElement.item(0).getTextContent();
			
			Allotjament allotjament = new Allotjament(
					a.getAttribute("tipus"),
					a.getElementsByTagName("nom").item(0).getTextContent(),
					new GPS(latitud, longitud),
					a.getElementsByTagName("acces").item(0).getTextContent(),
					adreca,
					a.getElementsByTagName("poblacio").item(0).getTextContent(),
					telf,
					mail
			);
			
			NodeList serveis = a.getElementsByTagName("serveis");
			if(serveis.getLength()>0){
				NodeList servList = serveis.item(0).getChildNodes();
				for(int j=0; j<servList.getLength(); j++){
					String clau = servList.item(j).getNodeName();
					if(!clau.equals("#text")){
						String valor= servList.item(j).getTextContent();
						if(valor.equals("")) valor = Allotjament.NO_DATA;
						allotjament.addServei(clau, valor);
						Log.d("servei", clau + ": " + valor);
					}
				}
			}
			
			via.addAllotjament(allotjament);
		}
		
		
	}

	private Drawable loadEsquema(String adress) throws Exception {
		Drawable d = null;
		try {
			URL url = new URL(adress);
			HttpURLConnection urlConnection = (HttpURLConnection) url.openConnection();
			InputStream in = new BufferedInputStream(urlConnection.getInputStream());
			d = Drawable.createFromStream(in, "esquemaDraw");
			in.close();
			urlConnection.disconnect();
		} catch (Exception e) {
			Log.e("RemoteData", "loadEsquema"+e.getMessage());
			throw e;
		}
		
		
		return d;
	}
	
	
	
	private Document getXmlDocument(InputStream in) throws Exception{
		Document doc = null;
		try{
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			
			doc = db.parse(in);
			doc.getDocumentElement().normalize();
		}catch(Exception e){
			Log.e("RemoteData", "getXmlDocument"+e.getMessage());
			throw e;
		}
		return doc;
	}
	
	
	
	private void parseZonaSector(Document doc, ArrayList<Zona> list) {
		NodeList zones = doc.getElementsByTagName("zona");
		for(int i=0; i<zones.getLength(); i++){
			Element ez = (Element)zones.item(i);
			Zona z = new Zona(ez.getAttribute("nom"));
			
			NodeList sectors = ez.getElementsByTagName("sector");
			for(int j=0; j<sectors.getLength(); j++){
				Element es = (Element)sectors.item(j);
				Sector s = new Sector(es.getAttribute("nom"));
				z.addComponent(s);
			}
			
			list.add(z);
		}
	}
	
	
	private void parseViaList(Document doc, ArrayList<Zona> list) {
		boolean hasZona=false;
		NodeList zones = doc.getElementsByTagName("zona");
		for(int iz=0; iz<zones.getLength(); iz++){
			Element ez = (Element)zones.item(iz);
			Zona z = new Zona(ez.getAttribute("nom"));
			
			boolean hasSector = false;
			NodeList sectors = ez.getElementsByTagName("sector");
			for(int is=0; is<sectors.getLength(); is++){
				Element es = (Element)sectors.item(is);
				Sector s = new Sector(es.getAttribute("nom"));
				
				boolean hasParet=false;
				NodeList parets = es.getElementsByTagName("paret");
				for(int ip=0; ip<parets.getLength(); ip++){
					Element ep = (Element)parets.item(ip);
					Paret p = new Paret(ep.getAttribute("nom"));
					
					NodeList vies = ep.getElementsByTagName("via");
					for(int iv=0; iv<vies.getLength(); iv++){
						Element ev = (Element)vies.item(iv);
						Element nom = (Element)ev.getElementsByTagName("nom").item(0);
						Element dm  = (Element)ev.getElementsByTagName("dmaxima").item(0);
						Element dmo = (Element)ev.getElementsByTagName("dobligada").item(0);
						Element metres  = (Element)ev.getElementsByTagName("metres").item(0);
						Via v = new Via(z.getName(), s.getName(), p.getName(), nom.getTextContent(), dm.getTextContent(), dmo.getTextContent(), Integer.parseInt(metres.getTextContent()));
						p.addComponent(v);
						hasParet=true;
					}
					
					if(hasParet){
						s.addComponent(p);
						hasSector=true;
					}
					hasParet=false;
				}
				
				
				if(hasSector) {
					z.addComponent(s);
					hasZona=true;
				}
				hasSector=false;
			}
			
			if(hasZona){
				list.add(z);
			}
			hasZona = false;
		}
	}
}
