package osc.petra.view;

import osc.petra.R;
import osc.petra.logic.ProgramControlInterface;
import osc.petra.logic.Query;
import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.CompoundButton.OnCheckedChangeListener;

public class ParametersQueryActivity extends PetraBaseActivity {
	
	private CheckBox checkEscalada;
	private CheckBox checkVia;
	private CheckBox checkDificultat;
	private CheckBox checkOrientacio;
	private CheckBox checkLlargs;
	private CheckBox checkRoca;
	
	private Spinner spinnerEscalada;
	private Spinner spinnerVia;
	private Spinner spinnerDificultat;
	private Spinner spinnerOrientacio;
	private Spinner spinnerLlargs;
	private Spinner spinnerRoca;
	
	private Button butSeguent;
	
	private Drawable drawableEscalada;
	private Drawable drawableVia;
	private Drawable drawableDificultat;
	private Drawable drawableOrientacio;
	private Drawable drawableLlargs;
	private Drawable drawableRoca;
	
	private static final int ACTIVAT = 255;
	private static final int DESACTIVAT = 100;
	
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.parameters_activity);
        
        ((PetraApplication)this.getApplication()).addActivity(this);
        
        this.butSeguent = (Button)this.findViewById(R.id.buttonSeguent);
        this.loadCheckBoxes();
        this.loadSpinners();
        
        this.butSeguent.setOnClickListener(new OnClickListener(){
			@Override
			public void onClick(View view) {
				ParametersQueryActivity.this.actionSeguent();
			}
        });
    }
    
    
    protected void actionSeguent() {
    	ProgramControlInterface pc = this.getProgramControl();
    	Query query = pc.getQuery();
    	
    	if(this.checkEscalada.isChecked()) query.setEscalada(((TextView)this.spinnerEscalada.getSelectedView()).getText().toString());
    	else query.setEscalada(Query.NO_DATA);
    	
    	if(this.checkVia.isChecked()){
    		String sVia =((TextView)this.spinnerVia.getSelectedView()).getText().toString();
    		if(sVia.equals("Equipada")) sVia = Query.EQUIPADA;
    		else if(sVia.equals("Semiequipada")) sVia = Query.SEMIEQUIPADA;
    		else sVia = Query.DESEQUIPADA;
    		query.setTipus(sVia);
    	}else query.setTipus(Query.NO_DATA);
    	
    	if(this.checkDificultat.isChecked()) {
    		CharSequence[] dif  = this.getResources().getTextArray(R.array.dificultat);
    		String sMaxDif =((TextView)this.spinnerDificultat.getSelectedView()).getText().toString();
    		String sDificultat = "";
    		boolean maxDif = false;
    		for(int i=0; i<dif.length && !maxDif; i++){
    			sDificultat += ":"+dif[i]+":";
    			if(dif[i].equals(sMaxDif)) maxDif=true;
    		}
    		Log.d("DIFICULTAT", sDificultat);
    		query.setDificultatMaxima(sDificultat);
    	}else query.setDificultatMaxima(Query.NO_DATA);
    	
    	if(this.checkOrientacio.isChecked()) query.setOrientacio(((TextView)this.spinnerOrientacio.getSelectedView()).getText().toString());
    	else query.setOrientacio(Query.NO_DATA);
    	
    	if(this.checkLlargs.isChecked()){
    		String sLlargs = ((TextView)this.spinnerLlargs.getSelectedView()).getText().toString();
    		if(sLlargs.equals("Un llarg")) sLlargs = Query.UN_LLARG;
    		else sLlargs = Query.DIVERSOS_LLARGS;
    		query.setLlargs(sLlargs);
    	}else query.setLlargs(Query.NO_DATA);
    	
    	if(this.checkRoca.isChecked()){
    		String sRoca = "Roca"+((TextView)this.spinnerRoca.getSelectedView()).getText().toString();
    		if(sRoca.equals("RocaGranítica")) sRoca="RocaGranitica";
    		query.setRoca(sRoca);
    	}else query.setRoca(Query.NO_DATA);
    	
    	Intent intent = new Intent(this, ViaListActivity.class);
    	this.startActivity(intent);
	}


	private void loadSpinners() {
		this.spinnerEscalada   = (Spinner)this.findViewById(R.id.spinner_escalada);
        this.spinnerVia        = (Spinner)this.findViewById(R.id.spinner_via);
        this.spinnerDificultat = (Spinner)this.findViewById(R.id.spinner_dificultat);
        this.spinnerOrientacio = (Spinner)this.findViewById(R.id.spinner_orientacio);
        this.spinnerLlargs     = (Spinner)this.findViewById(R.id.spinner_llargs);
        this.spinnerRoca       = (Spinner)this.findViewById(R.id.spinner_roca);
        
        this.drawableEscalada  = this.spinnerEscalada.getBackground();
        this.drawableVia       = this.spinnerVia.getBackground();
        this.drawableDificultat= this.spinnerDificultat.getBackground();
        this.drawableOrientacio= this.spinnerOrientacio.getBackground();
        this.drawableLlargs    = this.spinnerLlargs.getBackground();
        this.drawableRoca      = this.spinnerRoca.getBackground();
        
        this.drawableEscalada.setAlpha(DESACTIVAT);
        this.drawableVia.setAlpha(DESACTIVAT);
        this.drawableDificultat.setAlpha(DESACTIVAT);
        this.drawableOrientacio.setAlpha(DESACTIVAT);
        this.drawableLlargs.setAlpha(DESACTIVAT);
        this.drawableRoca.setAlpha(DESACTIVAT);
	}
    


	private void loadCheckBoxes() {
		this.checkEscalada   = (CheckBox)this.findViewById(R.id.checkBox_escalada);
        this.checkVia        = (CheckBox)this.findViewById(R.id.checkBox_via);
        this.checkDificultat = (CheckBox)this.findViewById(R.id.checkBox_dificultat);
        this.checkOrientacio = (CheckBox)this.findViewById(R.id.checkBox_orientacio);
        this.checkLlargs     = (CheckBox)this.findViewById(R.id.checkBox_llargs);
        this.checkRoca       = (CheckBox)this.findViewById(R.id.checkBox_roca);
        
        this.checkEscalada.setOnCheckedChangeListener(new OnCheckedChangeListener(){
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				ParametersQueryActivity.this.actionCheckBox(isChecked, spinnerEscalada, drawableEscalada);
			}
        });
        
        this.checkVia.setOnCheckedChangeListener(new OnCheckedChangeListener(){
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				ParametersQueryActivity.this.actionCheckBox(isChecked, spinnerVia, drawableVia);
			}
        });
        
        this.checkDificultat.setOnCheckedChangeListener(new OnCheckedChangeListener(){
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				ParametersQueryActivity.this.actionCheckBox(isChecked, spinnerDificultat, drawableDificultat);
			}
        });
        
        this.checkOrientacio.setOnCheckedChangeListener(new OnCheckedChangeListener(){
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				ParametersQueryActivity.this.actionCheckBox(isChecked, spinnerOrientacio, drawableOrientacio);
			}
        });
        
        this.checkLlargs.setOnCheckedChangeListener(new OnCheckedChangeListener(){
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				ParametersQueryActivity.this.actionCheckBox(isChecked, spinnerLlargs, drawableLlargs);
			}
        });
        
        this.checkRoca.setOnCheckedChangeListener(new OnCheckedChangeListener(){
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				ParametersQueryActivity.this.actionCheckBox(isChecked, spinnerRoca, drawableRoca);
			}
        });
	}
    
    
	protected void actionCheckBox(boolean isChecked, Spinner spinner, Drawable drawable) {
		spinner.setClickable(isChecked);
		
		if(isChecked) drawable.setAlpha(ACTIVAT);
		else drawable.setAlpha(DESACTIVAT);
		
		spinner.setBackgroundDrawable(drawable);
	}
	
}
