package osc.petra.view;

import osc.petra.R;
import osc.petra.dataModel.Parquing;
import osc.petra.dataModel.Via;
import osc.petra.logic.ProgramControlInterface;
import android.app.ProgressDialog;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TabHost;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;

public class ViaActivity extends PetraBaseActivity {
	
	private TabHost tabs;
	private TextView infoNom;
	private TextView infoParet;
	private TextView infoSector;
	private TextView infoZona;
	private TextView infoDM;
	private TextView infoDMO;
	private TextView infoEscalada;
	private TextView infoTipus;
	private TextView infoMetres;
	private TextView infoOrientacio;
	private TextView infoRoca;
	private TextView infoAncoratges;
	private TextView infoMaterial;
	private TextView parquingLatitud;
	private TextView parquingLongitud;
	private TextView parquingAcces;
	private TextView parquingDescripcio;
	private TextView viaLatitud;
	private TextView viaLongitud;
	private TextView viaAproximacio;
	private TextView viaDescens;
	private ImageView esquema;
	private ListView linkList;
	private ListView allotjamentList;
	
	private LinkListAdapter linkAdapter;
	private AllotjamentListAdapter allotjamentAdapter;
	
	ProgramControlInterface pc;
	
	
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.via_activity);
        
        this.obtainViewReferences();
        this.startTabs();
        
        this.linkAdapter = new LinkListAdapter(this);
		this.linkList.setAdapter(linkAdapter);
		this.linkList.setOnItemClickListener(new OnItemClickListener(){
			@Override
			public void onItemClick(AdapterView<?> parent, View view, int position,long id) {
				ViaActivity.this.actionLinkItem(position);
			}
        });
		
		this.allotjamentAdapter = new AllotjamentListAdapter(this);
		this.allotjamentList.setAdapter(allotjamentAdapter);
		
		
        this.esquema.setOnClickListener(new OnClickListener(){
			@Override
			public void onClick(View view) {
				ViaActivity.this.actionEsquema();
			}
        });
		
        this.pc = this.getProgramControl();
        if(this.pc.getViaActual().isCarregada()) this.showVia();
        else this.loadData();
        
    }
    
    
    @Override
    protected void onPause(){
    	super.onPause();
    	this.finish();
    }
    
    
	protected void actionEsquema() {
		Via via = this.getProgramControl().getViaActual();
		Uri uri = Uri.parse(via.getUrlEsquema());
		Intent intent = new Intent(Intent.ACTION_VIEW, uri);
		this.startActivity(intent);
	}



	protected void actionLinkItem(int position) {
		Uri uri = Uri.parse((String)this.linkAdapter.getItem(position));
		Intent intent = new Intent(Intent.ACTION_VIEW, uri);
		this.startActivity(intent);
	}



	protected void showVia() {
		
		Via via = this.pc.getViaActual();
		
		this.infoNom.setText(via.getNom());
		this.infoParet.setText(via.getParet());
		this.infoSector.setText(via.getSector());
		this.infoZona.setText(via.getZona());
		this.infoDM.setText(via.getDificultatM());
		this.infoDMO.setText(via.getDificultatO());
		this.infoEscalada.setText(via.getEscalada());
		this.infoTipus.setText(via.getTipus());
		this.infoMetres.setText(via.getMetres()+" m");
		this.infoOrientacio.setText(via.getOrientacio());
		this.infoRoca.setText(via.getRoca());
		
		String ancoratges = "";
		int length = via.equipamentNumber()-1;
		for(int i=0; i<length; i++){
			ancoratges+=via.getEquipament(i)+", ";
		}
		if(length>=0) ancoratges+=via.getEquipament(length)+".";
		else ancoratges = "---";
		this.infoAncoratges.setText(ancoratges);
		
		
		String material = "";
		length = via.materialNumber()-1;
		for(int i=0; i<length; i++){
			material += via.getMaterial(i)+", ";
		}
		if(length>=0) material += via.getMaterial(length)+".";
		else material = "---";
		this.infoMaterial.setText(material);
		
		Parquing p = via.getParquing();
		
		this.parquingLatitud.setText(""+p.getLocalitzacioGPS().getLatitud());
		this.parquingLongitud.setText(""+p.getLocalitzacioGPS().getLongitud());
		this.parquingAcces.setText(p.getAcces());
		this.parquingDescripcio.setText(p.getDescripcio());
		
		
		this.viaLatitud.setText(""+via.getLocalitzacioGPS().getLatitud());
		this.viaLongitud.setText(""+via.getLocalitzacioGPS().getLongitud());
		this.viaAproximacio.setText(via.getAproximacio());
		this.viaDescens.setText(via.getDescens());
		this.esquema.setImageDrawable(via.getEsquema());
		
		for(int i=0; i<via.linksNumber(); i++){
			this.linkAdapter.addLink(via.getLink(i));
		}
		
		for(int i=0; i<via.allotjamentsNumber(); i++){
			this.allotjamentAdapter.addAllotjament(via.getAllotjament(i));
		}
	}



	private void loadData() {
		Thread t = new Thread(){
			protected ProgressDialog progress = new ProgressDialog(ViaActivity.this);
			@Override
			public void run(){
				if(!ViaActivity.this.hasConnection()){
					ViaActivity.this.exceptionAction(ViaActivity.this.getString(R.string.necessitesInternet));
					return;
				}
				
				ViaActivity.this.runOnUiThread(new Runnable(){
					@Override
					public void run() {
						progress.setMessage(getString(R.string.loadData));
						progress.show();
					}
				});
				
				try {
					ViaActivity.this.pc.loadViaActual();
				} catch (Exception e) {
					ViaActivity.this.exceptionAction(e.getMessage());
				}
				
				ViaActivity.this.runOnUiThread(new Runnable(){
					@Override
					public void run() {
						ViaActivity.this.showVia();
						progress.dismiss();
					}
				});
			}
		};
		
		t.start();
	}
	
	
	
	
	private void obtainViewReferences() {
		this.infoNom            = (TextView)this.findViewById(R.id.infoNom);
		this.infoParet          = (TextView)this.findViewById(R.id.infoParet);
		this.infoSector         = (TextView)this.findViewById(R.id.infoSector);
		this.infoZona           = (TextView)this.findViewById(R.id.infoZona);
		this.infoDM             = (TextView)this.findViewById(R.id.infoDM);
		this.infoDMO            = (TextView)this.findViewById(R.id.infoDMO);
		this.infoEscalada       = (TextView)this.findViewById(R.id.infoEscalada);
		this.infoTipus          = (TextView)this.findViewById(R.id.infoVia);
		this.infoMetres         = (TextView)this.findViewById(R.id.infoMetres);
		this.infoOrientacio     = (TextView)this.findViewById(R.id.infoOrientacio);
		this.infoRoca           = (TextView)this.findViewById(R.id.infoRoca);
		this.infoAncoratges     = (TextView)this.findViewById(R.id.infoAncoratges);
		this.infoMaterial       = (TextView)this.findViewById(R.id.infoMaterial);
		this.parquingLatitud    = (TextView)this.findViewById(R.id.textViewlatitudParquing);
		this.parquingLongitud   = (TextView)this.findViewById(R.id.textViewLongitudParquing);
		this.parquingAcces      = (TextView)this.findViewById(R.id.textViewAccesParquing);
		this.parquingDescripcio = (TextView)this.findViewById(R.id.textViewDescripcioParquing);
		this.viaLatitud         = (TextView)this.findViewById(R.id.textViewLatitudVia);
		this.viaLongitud        = (TextView)this.findViewById(R.id.textViewLongitudVia);
		this.viaAproximacio     = (TextView)this.findViewById(R.id.textViewAproximacioVia);
		this.viaDescens         = (TextView)this.findViewById(R.id.textViewDescens);
		this.esquema            = (ImageView)this.findViewById(R.id.imageViewEsquema);
		this.linkList           = (ListView)this.findViewById(R.id.listViewLinks);
		this.allotjamentList    = (ListView)this.findViewById(R.id.listViewAllotjaments);
	}
	



	private void startTabs() {
		tabs = (TabHost)this.findViewById(R.id.tabhost);
        tabs.setup();
        
        TabHost.TabSpec spec = tabs.newTabSpec("tab1");
        spec.setContent(R.id.tab1);
        spec.setIndicator("Info Via");
        tabs.addTab(spec);
        
        spec = tabs.newTabSpec("tab2");
        spec.setContent(R.id.tab2);
        spec.setIndicator("Accés");
        tabs.addTab(spec);
        
        spec = tabs.newTabSpec("tab3");
        spec.setContent(R.id.tab3);
        spec.setIndicator("Esquema");
        tabs.addTab(spec);
        
        spec = tabs.newTabSpec("tab4");
        spec.setContent(R.id.tab4);
        spec.setIndicator("Allotjament");
        tabs.addTab(spec);
        
        spec = tabs.newTabSpec("tab5");
        spec.setContent(R.id.tab5);
        spec.setIndicator("Links");
        tabs.addTab(spec);
	}
}
