package osc.petra.view;

import java.util.List;

import osc.petra.R;
import osc.petra.dataModel.Via;
import osc.petra.logic.ProgramControlInterface;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.ExpandableListView;
import android.widget.TextView;
import android.widget.ExpandableListView.OnChildClickListener;

public class ViaListActivity extends PetraBaseActivity {
	
	private ExpandableListView listView;
	private ViaListAdapter listAdapter;
	private TextView textViewNoResult;
	
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.list_via);
        
        ((PetraApplication)this.getApplication()).addActivity(this);
        
        this.listView = (ExpandableListView)this.findViewById(R.id.expandableListView);
        this.textViewNoResult = (TextView)this.findViewById(R.id.textViewNoResult);
        
        this.listAdapter = new ViaListAdapter(this);
        this.listView.setAdapter(this.listAdapter);
        
        this.listView.setOnChildClickListener(new OnChildClickListener(){
			@Override
			public boolean onChildClick(ExpandableListView parent, View v, int indexGroup, int indexChild, long id) {
				ViaListActivity.this.actionSelectedVia(indexGroup, indexChild);
				return true;
			}
        });
        
        this.loadData();
    }
    
    
    
	private void loadData() {
		Thread t = new Thread(){
			protected ProgressDialog progress = new ProgressDialog(ViaListActivity.this);
			protected boolean dades = true;
			@Override
			public void run(){
				if(!ViaListActivity.this.hasConnection()){
					ViaListActivity.this.exceptionAction(ViaListActivity.this.getString(R.string.necessitesInternet));
					return;
				}
				
				
				ViaListActivity.this.runOnUiThread(new Runnable(){
					@Override
					public void run() {
						progress.setMessage(getString(R.string.loadData));
						progress.show();
					}
				});
				
				ProgramControlInterface pc = ViaListActivity.this.getProgramControl();
				
				try {
					pc.executeQuery();
				} catch (Exception e) {
					ViaListActivity.this.exceptionAction(e.getMessage());
				}
				
				List<String> zones = pc.getListZones();
				dades = (zones.size()>0);
				for(String z : zones){
					List<String> sectors = pc.getListSectors(z);
					for(String s : sectors){
						List<Via> vies = pc.getListVies(z, s);
						String zs = z + " - " + s;
						ViaListActivity.this.listAdapter.addViesSector(zs, vies);
					}
				}
				
				
				ViaListActivity.this.runOnUiThread(new Runnable(){
					@Override
					public void run() {
						ViaListActivity.this.listAdapter.notifyDataSetChanged();
						if(!dades) ViaListActivity.this.textViewNoResult.setVisibility(View.VISIBLE);
						else ViaListActivity.this.textViewNoResult.setVisibility(View.GONE);
						progress.dismiss();
					}
				});
			}
		};
		
		t.start();
	}



	protected void actionSelectedVia(int indexGroup, int indexChild) {
		ProgramControlInterface pc = this.getProgramControl();
		pc.setViaActual((Via)this.listAdapter.getChild(indexGroup, indexChild));
		
		Intent intent = new Intent(this, ViaActivity.class);
		this.startActivity(intent);
	}
}
