package osc.petra.view;

import java.util.List;

import osc.petra.R;
import osc.petra.logic.ProgramControlInterface;
import osc.petra.logic.QueryZona;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ExpandableListView;
import android.widget.ExpandableListView.OnChildClickListener;

public class ZonaListActivity extends PetraBaseActivity {
	
	private ExpandableListView listView;
	private ZonaListAdapter listAdapter;
	private Button butSeg;
	
	 /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.list_zona_sector);
        
        ((PetraApplication)this.getApplication()).addActivity(this);
        
        this.listView = (ExpandableListView)this.findViewById(R.id.expandableListView);
        this.butSeg   = (Button)this.findViewById(R.id.buttonSeguent);
        
        this.listAdapter = new ZonaListAdapter(this);
        this.listView.setAdapter(this.listAdapter);
        
        this.listView.setOnChildClickListener(new OnChildClickListener(){
			@Override
			public boolean onChildClick(ExpandableListView parent, View v, int indexGroup, int indexChild, long id) {
				ZonaListActivity.this.actionSelectChild(indexGroup, indexChild);
				return true;
			}
        });
        
        this.butSeg.setOnClickListener(new OnClickListener(){
			@Override
			public void onClick(View v) {
				ZonaListActivity.this.actionSeguent();
			}
        });
        
        this.loadData();
    }
    
    
    
    
	protected void actionSeguent() {
		ProgramControlInterface pc = this.getProgramControl();
		pc.newQuery(false);
		QueryZona query = (QueryZona)pc.getQuery();
		
		for(int i=0; i<this.listAdapter.getGroupCount(); i++){
			String zona = (String)this.listAdapter.getGroup(i);
			List<String> sectors = this.listAdapter.getSelectedSectors(i);
			if(sectors.size()>0)query.addZona(zona, sectors);
		}
		
		Intent intent = new Intent(this, ParametersQueryActivity.class);
		this.startActivity(intent);
	}




	protected void actionSelectChild(int indexGroup, int indexChild) {
		this.listAdapter.chageStateSector(indexGroup, indexChild);
		this.runOnUiThread(new Runnable(){
			@Override
			public void run() {
				ZonaListActivity.this.listAdapter.notifyDataSetChanged();
			}
		});
	}

	
	
	
	private void loadData() {
		Thread t = new Thread(){
			
			protected ProgressDialog progress = new ProgressDialog(ZonaListActivity.this);
			
			public void run(){
				if(!ZonaListActivity.this.hasConnection()){
					ZonaListActivity.this.exceptionAction(ZonaListActivity.this.getString(R.string.necessitesInternet));
					return;
				}
				
				ZonaListActivity.this.runOnUiThread(new Runnable(){
					@Override
					public void run() {
						progress.setMessage(getString(R.string.loadData));
						progress.show();
					}
				});
				
				ProgramControlInterface pc = ZonaListActivity.this.getProgramControl();
				
				try {
					pc.loadAllZonaAndSector();
				} catch (Exception e) {
					ZonaListActivity.this.exceptionAction(e.getMessage());
				}
				List<String> zones = pc.getListZones();
				
				for(String zona : zones){
					List<String> sectors = pc.getListSectors(zona);
					ZonaListActivity.this.listAdapter.addZonaSectors(zona, sectors);
				}
				
				
				ZonaListActivity.this.runOnUiThread(new Runnable(){
					@Override
					public void run() {
						ZonaListActivity.this.listAdapter.notifyDataSetChanged();
						progress.dismiss();
					}
				});
			}
		};
		t.start();
	}
}
